// Magma code to support the calculations in the paper: On some Generalized Fermat Equations of the form x^2+y^2l=z^p.
// This code verifies the computations of Lemma 7.3.

p:=17;
L<a>:=CyclotomicField(p);
K<w>:=sub< L | a+a^-1 >;
assert Degree(K) eq (p-1)/2;
OK:=Integers(K);

K4<c>:=sub< L | a+a^(3^4)+a^(3^8)+a^(3^12)>; // the unique subfield of K of degree 4.
assert Degree(K4) eq (p-1)/4;

F<b>:=QuadraticField(17);
assert IsSubfield(F,K4);

d:=Discriminant(MinimalPolynomial(w,K4));
S<Z>:=PolynomialRing(K4);
assert IsIsomorphic(K, ext<K4 | Z^2-d>);  // verification that K = K4(\sqrt(d))

X:=SmallModularCurve(20);
XK:=ChangeRing(X,K);
XK4:=ChangeRing(X,K4);
XF:=ChangeRing(X,F);

MordellWeilGroup(X);  // Z/6Z (true true)
MF,piF:=MordellWeilGroup(XF);  // Z/6Z + Z (true true)
MW,pi:=MordellWeilGroup(XK4);  // Z/6Z + Z  (true true), takes about 1 minute
R:= XK ! pi(MW.2);
Q:= XK ! pi(MW.1);

assert (XK4 ! piF(-MF.2-2*MF.1)) eq pi(MW.2); // generator of free part of XK4 comes from point in X(F), so XK4 = XF.
                                             
TorsXK:=TorsionSubgroup(XK); // Z/6Z

XdK4:=QuadraticTwist(XK4,d);
XdK:=ChangeRing(XdK4,K);

time Md,pid:=MordellWeilGroup(XdK4); // Z/2Z (true true), takes about 5 minutes

// We now verify 2-divisibility

qq:=137; 

q:=Factorisation(qq*OK)[1][1];
unif:=UniformizingElement(q);
Fq:=ResidueClassField(q);
Xq:=ChangeRing(XK,Fq);
Ab:=AbelianGroup(Xq);         // this is Z/2Z + Z/60Z

for b in [0,1] do 
    Pt:=R+b*Q;
    m:=Minimum([Valuation(a,q) : a in Eltseq(Pt) | not a eq 0]);
    Red1:=[unif^(-m)*a : a in Eltseq(Pt)];
    Red:=Xq![Evaluate(a,Place(q)) : a in Red1];   // reduction of Pt
    Order(Red);              // orders are 60, 60
end for;

