// Magma code to support the calculations in the paper: On some Generalized Fermat Equations of the form x^2+y^2l=z^p.

// This code verifies the computations of Lemma 7.4.

// This is the code of Anni and Siksek [1] (very slightly adapted).

i := 1; // Choose case

p:=17;
L<zet>:=CyclotomicField(p);
K<th>:=sub<L | zet+1/zet>;

if i eq 2 then 
   K:=Subfields(K,4)[1,1];
end if;

OK:=MaximalOrder(K);
assert NarrowClassNumber(K) eq 1;
U,phi:=UnitGroup(K);
// We will determine a system of totally positive units.
V:=U;
A:=AbelianGroup([2]);
for i in [1..Degree(K)] do
	img:=[];
	for u in OrderedGenerators(U) do
		if RealEmbeddings(phi(u))[i] gt 0 then
		   Append(~img,A!0);
		else
		   Append(~img,A!1);
		end if;
	end for;
	psi:=hom<U->A | img>;
	V:=V meet Kernel(psi);
end for;
posunits:=[phi(U!v) : v in OrderedGenerators(V)];
assert &and[IsTotallyPositive(u) : u in posunits];
assert &and[Norm(u) eq 1 : u in posunits];
print "The following is our system of totally positive units",
[K!u : u in posunits];
G,S,mu:=AutomorphismGroup(K);
subG:=[D`subgroup : D in Subgroups(G)];
BSet:={};
for D in subG do   // D is a subgroup of the Galois group.
	T,pi:=quo<G|D>; // T is the cosets of G/D.
	for Tp in Subsets({q : q in T}) do // Tp is T^prime
					// in the notation of the paper.
		if #Tp ne 0 and #Tp ne #T then
				// We check that Tp is non-empty and proper.
		   TpD:=[mu(t@@pi) : t in Tp];
				// TpD is the set of products
				// sigma tau 
				// where sigma is in D
				// tau is in T^prime.
		   BTpD:=GCD( [ Integers()!Norm(&*[u@t : t in TpD]-1)  : u in posunits  ]  );
				// This B_{T^prime,D}(u_1,...,u_d) in the
				// notation of the paper.
		   BSet:=BSet join {BTpD};
		end if;
	end for;
end for;
print "The set of B_{T^prime,D}(u_1,...,u_d) is ", BSet;
lset:=&cat[PrimeDivisors(B) : B in BSet];
lset:=[l : l in lset | l ge 5];  // We're only interested in ell >= 5.
print "The set of surviving primes ell is", lset;
